savedcmd_arch/x86/include/generated/asm/xen-hypercalls.h := sh 'scripts/xen-hypercalls.sh' arch/x86/include/generated/asm/xen-hypercalls.h include/xen/interface/xen-mca.h include/xen/interface/xen.h include/xen/interface/xenpmu.h
